def filter_isin(df, key, values):
    if df.empty:
        return df

    filtered = df.copy(deep=True)
    filtered = filtered[filtered[key].isin(values)]

    return filtered


def filter_startswith(df, key, values):
    if df.empty:
        return df

    filtered = df.copy(deep=True)
    filtered = filtered[filtered[key].str.startswith(values)]

    return filtered


def regressions(changes):
    try:
        data = changes.loc[changes["change"] == "regressions"].to_dict("records")
    except KeyError:
        return []

    regressions = {}
    for regression in data:
        key = (regression["environment"], regression["suite"])

        if key not in regressions:
            regressions[key] = []

        regressions[key].append(regression["test"])

    return regressions


def fixes(changes):

    try:
        data = changes.loc[changes["change"] == "fixes"].to_dict("records")
    except KeyError:
        return []

    fixes = {}
    for fix in data:
        key = (fix["environment"], fix["suite"])

        if key not in fixes:
            fixes[key] = []

        fixes[key].append(fix["test"])

    return fixes


def summary(results):
    data = results.to_dict("records")

    summary = {}
    for result in data:
        key = (result["environment"], result["suite"])

        if key not in summary:
            summary[key] = {"pass": 0, "fail": 0, "skip": 0, "xfail": 0}

        status = result["status"]
        summary[key][status] += 1

    return summary


def environments(results):
    return sorted(results.environment.unique().tolist())


def suites(results):
    return sorted(results.suite.unique().tolist())


def fails(results):
    data = results.loc[results["status"] == "fail"].to_dict("records")

    fails = {}
    for fail in data:
        key = (fail["environment"], fail["suite"])

        if key not in fails:
            fails[key] = []

        fails[key].append(fail["test"])

    return fails


def skips(results):
    data = results.loc[results["status"] == "skip"].to_dict("records")

    skips = {}
    for skip in data:
        key = (skip["environment"], skip["suite"])

        if key not in skips:
            skips[key] = []

        skips[key].append(skip["test"])

    return skips



import yaml
import pprint
import os.path
from squad_report.logging import getLogger

CONFIG_PATH = "~/.config/squad_report/config.yaml"

logger = getLogger(__name__)


class Config:
    squad_group = None
    squad_project = None
    email_from = "LKFT <lkft@linaro.org>"
    email_to = None
    email_cc = None
    email_subject = None
    reported_tested_by = "Linux Kernel Functional Testing <lkft@linaro.org>"
    signature_name = "Linaro LKFT"
    signature_url = "https://lkft.linaro.org"
    filter_suites = None
    filter_suite_prefixes = None
    filter_environments = None
    filter_environment_prefixes = None
    config_file = None
    send_on = None
    template = None
    output = None


def setup_config(args):
    yaml_dict = None
    config = Config()

    config.squad_group = args.group
    config.squad_project = args.project
    config.email_from = args.email_from
    config.email_to = args.email_to
    config.email_cc = args.email_cc
    config.email_subject = args.email_subject
    config.filter_suites = args.suites
    config.filter_suite_prefixes = args.suite_prefixes
    config.filter_environments = args.environments
    config.filter_environment_prefixes = args.environment_prefixes
    config.config_file = args.config or CONFIG_PATH
    config.send_on = args.send_on
    config.template = args.template
    config.output = args.output

    config.config_file = os.path.expanduser(config.config_file)

    if not os.path.exists(config.config_file) and args.config_report_type is None:
        logger.info("No config file")
    else:
        parser = yaml.safe_load
        with open(config.config_file, "r") as fh:
            try:
                yaml_dict = parser(fh)
            except yaml.YAMLError as e:
                raise SystemExit(e)

        for report in yaml_dict.get("report"):
            if report.get("name") == args.config_report_type:
                logger.debug(pprint.pformat(args.config_report_type))
                config.squad_group = config.squad_group or report.get("squad_group")
                config.squad_project = config.squad_project or report.get(
                    "squad_project"
                )
                config.email_from = config.email_from or report.get("email_from")
                config.email_to = config.email_to or report.get("email_to")
                config.email_cc = config.email_cc or report.get("email_cc", "")
                config.email_subject = config.email_subject or report.get(
                    "email_subject"
                )
                config.reported_tested_by = report.get("reported_tested_by")
                config.signature_name = report.get("sig_name")
                config.signature_url = report.get("sig_url")
                config.filter_suites = config.filter_suites or report.get(
                    "filter_suites"
                )
                config.filter_suite_prefixes = (
                    config.filter_suite_prefixes or report.get("filter_suite_prefixes")
                )
                config.filter_environments = config.filter_environments or report.get(
                    "filter_environments"
                )
                config.filter_environment_prefixes = (
                    config.filter_environment_prefixes
                    or report.get("filter_environment_prefixes")
                )
                config.send_on = config.send_on or report.get("send_on")
                config.template = report.get("template")
                config.output = config.output or report.get("output")

                logger.debug(pprint.pformat(config))
    return config



import jinja2
import requests.compat
import squad_report.dataframe
from squad_client.core.api import SquadApi
from os.path import expanduser, dirname, join

te = jinja2.Environment(
    extensions=["jinja2.ext.loopcontrols"],
    loader=jinja2.ChoiceLoader(
        [
            jinja2.FileSystemLoader(join(dirname(__file__), "templates")),
            jinja2.FileSystemLoader(expanduser("~/.config/squad_report/templates")),
        ]
    ),
    trim_blocks=True,
    lstrip_blocks=True,
    undefined=jinja2.StrictUndefined,
)


def test_results(results):
    summary = squad_report.dataframe.summary(results)
    passing = failing = skiping = xfailing = 0
    for key in summary:
        if key[1] != "build":
            passing = passing + summary[key].get("pass")
            failing = failing + summary[key].get("fail")
            skiping = skiping + summary[key].get("skip")
            xfailing = xfailing + summary[key].get("xfail")
    total = passing + failing + skiping + xfailing
    test_result = {
        "total": total,
        "pass": passing,
        "fail": failing,
        "skip": skiping,
        "xfail": xfailing,
    }
    return test_result


def create_report(config, group, project, build, base_build, changes, results):
    args = {
        "build": build,
        "build_url": squad_build_url(group, project, build),
        "base_build": base_build,
        "regressions": squad_report.dataframe.regressions(changes),
        "fixes": squad_report.dataframe.fixes(changes),
        "test_result": test_results(results),
        "summary": squad_report.dataframe.summary(results),
        "environments": squad_report.dataframe.environments(results),
        "suites": squad_report.dataframe.suites(results),
        "fails": squad_report.dataframe.fails(results),
        "skips": squad_report.dataframe.skips(results),
        "total_tests": len(results),
        "config": config,
    }

    if config.email_subject:
        config.email_subject = te.from_string(config.email_subject).render(**args)

    # generate = True means always generate the report.
    generate = True
    if config.email_subject and config.send_on:
        if "FAIL" in config.send_on and len(args["fails"]) == 0:
            generate = False
        if "REG" in config.send_on and len(args["regressions"]) == 0:
            generate = False
    # all
    if generate:
        return te.get_template(config.template + ".txt.jinja").render(**args)

    return None


def squad_build_url(group, project, build):
    return requests.compat.urljoin(
        SquadApi.url, "/".join([group.slug, project.slug, "build", build.version])
    )
